local HighScores = LibStub:NewLibrary("SomeTracker-HighScores", 1)
if not HighScores then return end

local _, month, day, year = CalendarGetDate()
local ready = year and month and ((year > 2017) or (year == 2017 and month >= 4))
HighScores.ready = ready

local D = LibStub("DoomCore-1.6")
local scores
local options

local function clearScores()
  local handler = options.handler
  handler.core.highscores = {}
  HighScores:Load(handler, handler.core.highscores)
end

local function resetOptions()
  options = {
  get = "ConfGetDebug",
  set = "ConfSetDebug",
  type = "group",
  args = {
    aprilfools = {
      type = "description",
      name = "Happy April Fool's Day!",
      width = "full",
      fontSize = "medium",
      hidden = true,
      order = 0
    },
    output = {
      type = "description",
      name = "Output:",
      width = "half",
      fontSize = "medium",
      order = 10
    },
    scoreChat = {
      type = "toggle",
      name = "Chat",
      width = "half",
      order = 11
    },
    scoreIcon = {
      type = "toggle",
      name = "Flash icon",
      order = 12
    },
    space = {
      type = "description",
      name = "",
      order = 13
    },
    clear = {
      func = clearScores,
      type = "execute",
      name = "Delete all",
      order = 14
    },
    sep = {
      type = "header",
      name = "Scores",
      width = "full",
      order = 20
    }
  }
  }
end
resetOptions()

local lib = {}
lib.registry = LibStub("AceConfigRegistry-3.0")
lib.settings = LibStub("AceConfigDialog-3.0")

if ready then lib.registry:RegisterOptionsTable("Some Tracker: High Scores", options) end

function HighScores:Display(name, score)
  options.args[name .. "_label"] = {
    type = "description",
    name = D.colorify(name, score.col),
    fontSize = "medium",
    image = tostring(score.icon),
    imageWidth = 16,
    imageHeight = 16,
    width = "double",
    order = 100 + (1000000000 - score.amount) + D.orderNum(name)
  }
  options.args[name] = {
    type = "description",
    fontSize = "medium",
    name = D.colorify(D.roundString(score.amount), score.col),
    order = 100.001 + (1000000000 - score.amount) + D.orderNum(name),
    width = "half"
  }
end

function HighScores:Load(handler, newDB)
  resetOptions()
  if month == 4 and day == 1 and year == 2017 then
    options.args.aprilfools.hidden = false
    if not handler:ConfGetDebug{ "fooled" } then
      handler:ConfSetDebug({ "fooled" }, true)
      handler:ConfSetDebug({ "scoreChat" }, true)
      handler:ConfSetDebug({ "scoreIcon" }, true)
    end
  elseif handler:ConfGetDebug{ "fooled" } then
    handler:ConfSetDebug{ "fooled" }
    handler:ConfSetDebug{ "scoreChat" }
    handler:ConfSetDebug{ "scoreIcon" }
  end
  options.handler = handler
  if ready then lib.registry:RegisterOptionsTable("Some Tracker: High Scores", options) end
  scores = newDB

  for scoreName, score in pairs(scores) do
    self:Display(scoreName, score)
  end
end

local prepatch = select(4,GetBuildInfo()) == 70100
function HighScores:Score(name, amount, icon, col)
  if prepatch then return end
  if not scores then return end
  if scores[name] and scores[name].amount > amount then return end
  scores[name] = { amount = amount, icon = icon, col = col }
  self:Display(name, scores[name])
  if options.handler:ConfGetDebug{ "scoreChat" } then
    print(D.colorify("New high score of " .. D.roundString(amount) .. " for |T" .. icon .. ":TextHeight|t" .. name .. "!","00ff00"))
  end
  return true
end

function HighScores:GetScore(name)
  if not scores or not name then return end
  local score = scores[name]
  if not score then return 0 end
  return score.amount
end
